using System.Collections.Concurrent;
using System.Globalization;
using System.Text.RegularExpressions;

using Microsoft.AspNetCore.Components;
using Microsoft.AspNetCore.Components.Rendering;
using Microsoft.AspNetCore.Components.Web;
using Microsoft.AspNetCore.Html;
using Microsoft.AspNetCore.Mvc.ViewFeatures;

namespace NSeguin.Dev.Web.Components.Shared;

public partial class HeroIcon : ComponentBase
{
    private const string PathBase = "wwwroot/lib/heroicons";
    private static readonly ConcurrentDictionary<string, string> Cache = new();
    private readonly Lazy<string> _rawSvg;
    private readonly Lazy<string> _svg;

    public HeroIcon()
    {
        _rawSvg = new Lazy<string>(GetRawSvg);
        _svg = new Lazy<string>(GetSvg);
    }

    [Parameter]
    public string Name { get; set; }

    [Parameter]
    public bool UseSmallIcon { get; set; } = false;

    [Parameter]
    public bool UseOutline { get; set; } = true;

    [Parameter]
    public string? Class { get; set; }

    private string Svg => _svg.Value;

    private string FileName =>
        Path.Combine(
            PathBase,
            UseSmallIcon ? "20" : "24",
            UseOutline ? "outline" : "solid",
            Name + ".svg");

    protected override void BuildRenderTree(RenderTreeBuilder builder)
    {
        builder.AddMarkupContent(0, Svg);
    }

    [GeneratedRegex("""<svg\s+(?:class=\s*"[^"]*"\s+)?""")]
    private static partial Regex SvgRegex();

    private string GetSvg()
    {
        if (Class is null)
        {
            return _rawSvg.Value;
        }

        var svg = SvgRegex().Replace(_rawSvg.Value, $"""<svg class="{Class}" """);
        return svg;
    }

    private string GetRawSvg() => Cache.GetOrAdd(FileName, File.ReadAllText);
}
